/* ++++++++++++++++++++++++++++++++++++++++++++++   
   PROGRAM TO MAKE AND RECORD PICTURES ON LDR
   WITH INTERPOLATIONS.
   Pablo Laguna
   March 20,1990
   With diagnostics, improvements, and other hacks 
   added by Robert Jacob 11/90
   Changes: Singlebuffer, draws polygons as it finds them,
   elimination of POLYGON structure,more comments written to
   standard out.
   ++++++++++++++++++++++++++++++++++++++++++++++ */

/* +++  Libraries +++ */

#include <math.h>
#include <gl.h>
#include <stdio.h>
#include <device.h>
#include <sys/types.h>
#include <termio.h>
#include <fcntl.h>
#include <sys/signal.h>

/* +++ Material Definitions +++ */

float material_1[] = {
	EMISSION,  0.0, 0.0, 0.0,
	AMBIENT,   0.9, 0.0, 0.0,   
	DIFFUSE,   0.9, 0.0, 0.0,   
	SPECULAR,  0.9, 0.0, 0.0,   
	SHININESS, 200.0,            
	ALPHA,     1.0,
	LMNULL};

float material_2[] = {
	EMISSION,  0.0, 0.0, 0.0,
	AMBIENT,   0.0, 0.0, 0.0,   
	DIFFUSE,   0.0, 0.0, 0.0,   
	SPECULAR,  0.0, 0.0, 0.0,   
	SHININESS, 0.0,            
	ALPHA,     0.0,
	LMNULL};

float material_3[] = {
	EMISSION,  0.0, 0.08,0.5,
	AMBIENT,  0.4, 0.6, 0.8,   
	DIFFUSE,   0.4,0.6, 0.8,   
	SPECULAR,  0.0, 0.0, 0.0,   
	SHININESS, 0.0,            
	ALPHA,     0.4,
	LMNULL};

float glass_material[] = {
	EMISSION,  0.0, 0.0, 0.0,
	AMBIENT,   1.0, 0.0, 1.0,   
	DIFFUSE,   1.0, 0.0, 1.0,   
	SPECULAR,  1.0, 0.0, 1.0,   
	SHININESS, 128.0,            
	ALPHA,     0.08,
	LMNULL};

float light_1[] = {
	AMBIENT,   0.2, 0.2, 0.2,
	LCOLOR,    0.8, 0.8, 0.8,
	POSITION,  1.0, 1.0, 1.0, 0.0,
	LMNULL};

float ambient_1[] = {
	AMBIENT,     0.2, 0.2, 0.2,
	LOCALVIEWER, 0.0,
	ATTENUATION, 1.0, 1.0,
	LMNULL};


/* +++ Function Definitions +++ */

#define MAX(X,Y)    ( (X) > (Y) ? (X) : (Y) )
#define MIN(X,Y)    ( (X) < (Y) ? (X) : (Y) )
#define ABS(X)      ( (X) < 0 ? -(X) : (X) )

/* +++ Constant Definitions +++ */

#define STDIN 0
#define STDOUT 1
#define STDERR 2

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define ZOOM 64            /* Iterations in zoom */ 
#define JP 64              /* Picture dimension size */
#define N_CONT 2           /* No. contours per frame */
#define N_INTER 1          /* No. of interpolations */

/* +++ Structure Definitions +++ */

typedef struct {
	Coord x;
	Coord y;
	Coord z;
} POINT;

typedef struct {
	POINT esquina[8];
	POINT donde[6];
} BOX;


typedef struct {
	int head;
	int tail;
} EDGE;

typedef struct {
	POINT esquina;
} CORNER;

struct LINK {                      /* cell link structure */
	int label_no;               /* link label */
	int on_off;                 /* if poly cuts link, ON */
	struct LIST *to_list;       /* pointer to next list */
};

struct LIST {                      /* link ordered list */
	struct LINK label[4];
};

/* +++ Pointers ++ */

BOX          *caja;         /* pointer to box */
FILE         *tp_01;        /* pointer to tape 01 */
FILE         *tp_02;        /* pointer to tape 02 */

FILE         *tp_03;        /* pointer to tape 03 */

EDGE         *eg[12];       /* pointer to edge list */
CORNER       *ez[8];        /* pointer to corner list */
struct LIST  *pl[6];        /* pointer to vertex list */

/* +++ Global Data declarations +++ */

Matrix idmat = {
	1.0, 0.0, 0.0, 0.0,   /* identity matrix */
	0.0, 1.0, 0.0, 0.0,
	    0.0, 0.0, 1.0, 0.0,
	    0.0, 0.0, 0.0, 1.0};

int ttfile;
int t_p[3];                    /* No. of polygon per contour */

int c_lbl[3];                  /* Contour labels */
int pict_r;                    /* pict read */
int pict_w;                    /* pict done */
int pict_1;                    /* first set of pictures */

float pict_int[JP][JP][JP];    /* picture array */
float pict_old[JP][JP][JP];    /* old picture array */
float pict_new[JP][JP][JP];    /* new picture array */

char in_file[40];              /* input file name */

/* +++ Main Program +++ */

/************************************************
** Driver of the program 
************************************************/
main()
{
	int i, j;                      /* counter */
	int reloj;

	read_to_restart();
	set_up_tools();
	tp_02 = fopen(in_file,"rb");
	if (pict_1)
	{
		read_data(pict_old);
		pict_r++;
		pict_1 = 0;
	}
	else 
	{
		if ((tp_03 = fopen("lastpict","rb")) == NULL)
		{
			printf(" Lastpict file does not exist \n ");
			exit();
		}
		printf("Reading Lastpict file. . .");
		fread(pict_old,sizeof(pict_old),1,tp_03);
		printf("Done\n");
		pict_r++;
		fclose(tp_03);
	}
	for (;;) {
		read_data(pict_new);
		if (feof(tp_02)) goto out_of_data;
		for (j = 0; j < N_INTER; j++) {
			reloj = time(0);
			find_interpol(j);
			make_picture();
			/*display_picture();*/
			pict_w++;
			reloj = time(0)-reloj;
			sleep(30);
			printf(" Picture (%d,%d) done in %d seconds.\n",pict_r,j,reloj);
		}
		pict_r++;
		cp_new_to_old(pict_new,pict_old);
	}
out_of_data: 
	printf(" Pictures done = %d \n",pict_w);
	tp_03 = fopen("lastpict","wb");
	fwrite(pict_old,sizeof(pict_old),1,tp_03);
	fclose(tp_03);
	save_to_restart();
	blanktime(60300);
}

/* +++ Subroutines +++ */

/*************************************************
** Set up tools 
*************************************************/
set_up_tools()
{
	define_lists();
	define_esquinas();
	define_edges();
	make_box();
	graph_environment();
	def_light_calc();
	use_light_calc();
}


/********************************************
** Localize and define polygons in cells
** that define contour label. 
*********************************************/
make_picture()
{
	int i, j, k;                 /* cell coordinates */
	int p_i;			/*polygon counter */
	int c_i;                     /* contour counter */
	int l_i;                     /* link label */

	int v_i;                     /* vertex label */

	int v_max;               /* max no. of vertices */
	int cor_i;                   /* corner label */

	int v_list[12];              /* vertex label list */

	int cor_max, cor_min;        /* index of max and min corner values */
	int h_l, t_l;                /* head and tail link corner labels */

	Coord x_d = 2.7;
	Coord y_d = 1.9;
	Coord z_d = 2.5;
	Coord x_v, y_v, z_v;

	float angulo;
	float d_fact;

	char stx = 0x02;
	char etx = 0x03;

	float LABEL;               /* Contour label (float) */
	float tt[12][3];           /* vertex temporary storage */
	float vv[3];               /* vertex temporary storage */
	float above, below;        /* max and min corner values in cell */
	float corner[8];           /* picture corner values */
	float h_v, t_v;            /* link head and tail picture values */
	float delta;               /* interpolation distance */
	float aa[3], bb[3];        /* reference vector */
	float ref_vect[3];         /* reference vector */
	float norm_vect[3];        /* temp normal vector */
	float mas = -1.0;          /* large values (inside,outside) = (-1,+1) */
	float Scale = 3.0/(JP-1);  /* scale factor */
	float off_set = (JP-1)/2;  /* off set distance */



	d_fact = 1+0.5*(1-(float)(MIN(pict_w,ZOOM))/(float)(ZOOM));
	x_v = x_d * d_fact;
	y_v = y_d * d_fact;
	z_v = z_d * d_fact;
	czclear(0x00000000,0x7fffff);  /* clear to black and set zbuffer */
	pushmatrix();
	angulo = -90.0 + pict_w * 0.1;
	lookat(x_v,y_v,z_v,0.0,0.0,0.0,0);
	scale(1.1,1.1,1.1);
	rot(angulo,'Y');

	/* Sweep each contour label */
	for (c_i = 0; c_i < N_CONT; c_i++) {
		printf("Sweeping for contour level %d\n",c_lbl[c_i]);
		/* Compute contour label value */
		lmbind(MATERIAL,c_i + 1);
		LABEL = 1.0000045643 * (float) c_lbl[c_i] * 2.55;

		/* Sweep each cell */

		t_p[c_i] = 0;

		for (i = 0; i < JP-1; i++) {
			for (j = 0; j < JP-1; j++) {
				for (k = 0; k < JP-1; k++) {
					/* Define corner values */

					corner[0] = pict_int[i+0][j+0][k+0];
					corner[1] = pict_int[i+1][j+0][k+0];
					corner[2] = pict_int[i+0][j+1][k+0];
					corner[3] = pict_int[i+1][j+1][k+0];
					corner[4] = pict_int[i+0][j+0][k+1];
					corner[5] = pict_int[i+1][j+0][k+1];
					corner[6] = pict_int[i+0][j+1][k+1];
					corner[7] = pict_int[i+1][j+1][k+1];

					/* Find max and min values in the cell */

					above = -1.0e+16;
					below =  1.0e+16;
					for (cor_i = 0; cor_i < 8; cor_i++) {
						above = MAX(above,corner[cor_i]);
						below = MIN(below,corner[cor_i]);
					}

					/* No polys in cell ? */

					if ((above < LABEL) || (below > LABEL)) continue;

					/* Find indices of max and min values at the cell */

					above = -1.0e+16;
					below =  1.0e+16;
					for (cor_i = 0; cor_i < 8; cor_i++) {
						if (corner[cor_i] > above) {
							above = corner[cor_i];
							cor_max = cor_i;
						}
						if (corner[cor_i] < below) {
							below = corner[cor_i];
							cor_min = cor_i;
						}
					}

					/* Find links where contour cuts */
					/* reset array */
					v_i=0;
					for(v_i=0;v_i<12;v_i++)v_list[v_i]=0.0;
					v_i = 0;
					for (l_i = 0; l_i < 12; l_i++) {
						h_l = eg[l_i]->head;
						t_l = eg[l_i]->tail;
						h_v = corner[h_l];
						t_v = corner[t_l];
						if ((MAX(h_v,t_v) > LABEL) && (MIN(h_v,t_v) < LABEL)) {
							delta = (LABEL - t_v)/(h_v - t_v);
							vv[0] = (ez[h_l]->esquina.x - ez[t_l]->esquina.x)*delta;
							vv[1] = (ez[h_l]->esquina.y - ez[t_l]->esquina.y)*delta;
							vv[2] = (ez[h_l]->esquina.z - ez[t_l]->esquina.z)*delta;
							vv[0] = i + ez[t_l]->esquina.x + vv[0];
							vv[1] = j + ez[t_l]->esquina.y + vv[1];
							vv[2] = k + ez[t_l]->esquina.z + vv[2];
							tt[l_i][0] = Scale * (vv[0] - off_set);
							tt[l_i][1] = Scale * (vv[1] - off_set);
							tt[l_i][2] = Scale * (vv[2] - off_set);
							v_list[v_i] = l_i;
							v_i++;
						}
					}

					/* No contour polygons in cell */

					if (v_i < 3) continue;
					/* Sort poly vertex list */

					sort_links(v_list, v_i);

					/* Find reference vector which points from max value to min */

					ref_vect[0] = mas*(ez[cor_max]->esquina.x - ez[cor_min]->esquina.x);
					ref_vect[1] = mas*(ez[cor_max]->esquina.y - ez[cor_min]->esquina.y);
					ref_vect[2] = mas*(ez[cor_max]->esquina.z - ez[cor_min]->esquina.z);

					/* Use the first three vertices of the polygon to construct
** the vectors defining the poly surface */

					aa[0] = tt[v_list[0]][0]-tt[v_list[1]][0];
					aa[1] = tt[v_list[0]][1]-tt[v_list[1]][1];
					aa[2] = tt[v_list[0]][2]-tt[v_list[1]][2];
					bb[0] = tt[v_list[2]][0]-tt[v_list[1]][0];
					bb[1] = tt[v_list[2]][1]-tt[v_list[1]][1];
					bb[2] = tt[v_list[2]][2]-tt[v_list[1]][2];

					/* Get normal to the polygon */

					get_normal(aa, bb, ref_vect, norm_vect);

					/* Draw the polygon */

					bgnpolygon();
					n3f(norm_vect);
					for ( cor_i = 0; cor_i < MIN(8,v_i); cor_i++ ) 
						v3f(tt[v_list[cor_i]]);
					endpolygon();

					t_p[c_i]++;
				}  /* end of k loop */
			}  /* end of j loop */
		}  /* end of i loop */
		printf("Done\n");
		printf("I drew %d polygons\n",t_p[c_i]);
	}  /* end of contour level loop */
	glass_box();
	popmatrix();
	printf ("\n View (%5.3f,%5.3f,%5.3f) Rot (%3.2f)",
               x_v,y_v,z_v,angulo);
}

/***********************************************************
** Given two vectors, this function computes the normal to
** the surface that those vector define.
************************************************************/
get_normal(a_vect, b_vect, r_vect, n_vect)
float a_vect[], b_vect[];        /* input vectors */
float n_vect[];                  /* normal vector */
float r_vect[];                  /* reference vector */
{
	float norma;                  /* vector magnitude */
	float dot;                    /* dot product */

	/* Compute the cross pruduct of input vectors */

	n_vect[0] = a_vect[1] * b_vect[2]
	    - a_vect[2] * b_vect[1];
	n_vect[1] = a_vect[2] * b_vect[0]
	    - a_vect[0] * b_vect[2];
	n_vect[2] = a_vect[0] * b_vect[1]
	    - a_vect[1] * b_vect[0];

	/* Normalize unit normal */

	norma = sqrt( pow(n_vect[0],2)
	    + pow(n_vect[1],2)
	    + pow(n_vect[2],2) );

	if (norma == 0) {
		printf("\n\t !!! Normal vanishes !!!\n");
		exit();
	}

	n_vect[0] = n_vect[0]/norma;
	n_vect[1] = n_vect[1]/norma;
	n_vect[2] = n_vect[2]/norma;

	/* Compute the dot pruduct of the normal with reference vector */

	dot = n_vect[0] * r_vect[0]
	    + n_vect[1] * r_vect[1]
	    + n_vect[2] * r_vect[2];

	if (dot < 0.0) {
		n_vect[0] = -1.0*n_vect[0];
		n_vect[1] = -1.0*n_vect[1];
		n_vect[2] = -1.0*n_vect[2];

	}
}

/**************************************************
** Function to define the edge structure.
***************************************************/
define_edges()
{
	int i;                 /* counter */

	for (i = 0; i < 12; i++) eg[i] = (EDGE *)malloc(sizeof(EDGE));

	/* Edge no. 0 */

	eg[0]->tail = 0;
	eg[0]->head = 1;

	/* Edge no. 1 */

	eg[1]->tail = 2;
	eg[1]->head = 3;

	/* Edge no. 2 */

	eg[2]->tail = 6;
	eg[2]->head = 7;

	/* Edge no. 3 */

	eg[3]->tail = 4;
	eg[3]->head = 5;

	/* Edge no. 4 */

	eg[4]->tail = 0;
	eg[4]->head = 2;

	/* Edge no. 5 */

	eg[5]->tail = 4;
	eg[5]->head = 6;

	/* Edge no. 6 */

	eg[6]->tail = 5;
	eg[6]->head = 7;

	/* Edge no. 7 */

	eg[7]->tail = 1;
	eg[7]->head = 3;

	/* Edge no. 8 */

	eg[8]->tail = 0;
	eg[8]->head = 4;

	/* Edge no. 9 */

	eg[9]->tail = 1;
	eg[9]->head = 5;

	/* Edge no. 10 */

	eg[10]->tail = 3;
	eg[10]->head = 7;

	/* Edge no. 11 */

	eg[11]->tail = 2;
	eg[11]->head = 6;


}

/***************************************************
** Defines corner coordinates.
****************************************************/
define_esquinas()
{
	int i;                 /* counter */

	for (i = 0; i < 8; i++) ez[i] = (CORNER *)malloc(sizeof(CORNER));

	/* Corner no. 0 */

	ez[0]->esquina.x = 0;
	ez[0]->esquina.y = 0;
	ez[0]->esquina.z = 0;

	/* Corner no. 1 */

	ez[1]->esquina.x = 1;
	ez[1]->esquina.y = 0;
	ez[1]->esquina.z = 0;

	/* Corner no. 2 */

	ez[2]->esquina.x = 0;
	ez[2]->esquina.y = 1;
	ez[2]->esquina.z = 0;

	/* Corner no. 3 */

	ez[3]->esquina.x = 1;
	ez[3]->esquina.y = 1;
	ez[3]->esquina.z = 0;

	/* Corner no. 4 */

	ez[4]->esquina.x = 0;
	ez[4]->esquina.y = 0;
	ez[4]->esquina.z = 1;

	/* Corner no. 5 */

	ez[5]->esquina.x = 1;
	ez[5]->esquina.y = 0;
	ez[5]->esquina.z = 1;

	/* Corner no. 6 */

	ez[6]->esquina.x = 0;
	ez[6]->esquina.y = 1;
	ez[6]->esquina.z = 1;

	/* Corner no. 7 */

	ez[7]->esquina.x = 1;
	ez[7]->esquina.y = 1;
	ez[7]->esquina.z = 1;


}

/***************************************************************
** Given the list of vertex of the contour polygon,
** sort_links returns the correct order for its vertex points.
****************************************************************/
sort_links(old_list, vertex_max)
int vertex_max;                   /* max. no. of vertex in polygon */
int old_list[];                   /* list to be ordered */
{
	int list_i;                    /* list counter */
	int label_i;                   /* label counter */
	int v_i;                       /* vertex counter */
	int new_list[12];              /* list in order */
	int ON = 1;                    /* poly vertex cuts cell vertex */
	int OFF = 0;                   /* poly vertex does not cut cell vertex */

	struct LIST *p_list;           /* pointer to vertex list */


	/* allocate memory por vertex list */

	p_list = (struct LIST *) malloc(sizeof(struct LIST));

	/* Turn all labels OFF */

	for (list_i = 0; list_i < 6; list_i++) {
		for (label_i = 0; label_i < 4; label_i++) {
			pl[list_i]->label[label_i].on_off = OFF;
		}
	}

	/* Turn ON only elements to be ordered */

	for (v_i = 0; v_i < vertex_max; v_i++) {
		for (list_i = 0; list_i < 6; list_i++) {
			for (label_i = 0; label_i < 4; label_i++) {
				if (pl[list_i]->label[label_i].label_no == old_list[v_i])
					pl[list_i]->label[label_i].on_off = ON;
			}
		}
	}

	/* Find a list for old_list[0] to start sorting */

	for (list_i = 0; list_i < 6; list_i++) {
		for (label_i = 0; label_i < 4; label_i++) {
			if (pl[list_i]->label[label_i].label_no == old_list[0]) {
				new_list[0] = pl[list_i]->label[label_i].label_no;
				p_list = pl[list_i];
			}
		}
	}

	/* Let's sort ... */

	for (v_i = 1; v_i < vertex_max; v_i++) {
		for (label_i = 0; label_i < 4; label_i++) {
			if ((p_list->label[label_i].on_off == ON) &&
			    (p_list->label[label_i].label_no != 
			    new_list[v_i - 1])) {
				new_list[v_i] = p_list->label[label_i].label_no;
				p_list = p_list->label[label_i].to_list;
				break;
			}
		}
	}
	free(p_list);

	/* Rewrite input vertex list */

	for (v_i = 1; v_i < vertex_max; v_i++)
		old_list[v_i] = new_list[v_i];
}

/******************************************************
** Function to define the lists of cell links
** for sorting polygon vertex.
*******************************************************/
define_lists()
{
	int list_i;               /* list counter */

	/* Allocate memory for each pointer */
	for (list_i = 0; list_i < 6; list_i++)
		pl[list_i] = (struct LIST *) malloc(sizeof(struct LIST));

	/* Define lists elements */

	/* List x = 0 */

	pl[0]->label[0].label_no = 8;
	pl[0]->label[0].to_list = pl[2];
	pl[0]->label[1].label_no = 4;
	pl[0]->label[1].to_list = pl[4];
	pl[0]->label[2].label_no = 5;
	pl[0]->label[2].to_list = pl[5];
	pl[0]->label[3].label_no = 11;
	pl[0]->label[3].to_list = pl[3];

	/* List x = 1 */

	pl[1]->label[0].label_no = 9;
	pl[1]->label[0].to_list = pl[2];
	pl[1]->label[1].label_no = 7;
	pl[1]->label[1].to_list = pl[4];
	pl[1]->label[2].label_no = 6;
	pl[1]->label[2].to_list = pl[5];
	pl[1]->label[3].label_no = 10;
	pl[1]->label[3].to_list = pl[3];

	/* List y = 0 */

	pl[2]->label[0].label_no = 0;
	pl[2]->label[0].to_list = pl[4];
	pl[2]->label[1].label_no = 8;
	pl[2]->label[1].to_list = pl[0];
	pl[2]->label[2].label_no = 9;
	pl[2]->label[2].to_list = pl[1];
	pl[2]->label[3].label_no = 3;
	pl[2]->label[3].to_list = pl[5];

	/* List y = 1 */

	pl[3]->label[0].label_no = 1;
	pl[3]->label[0].to_list = pl[4];
	pl[3]->label[1].label_no = 11;
	pl[3]->label[1].to_list = pl[0];
	pl[3]->label[2].label_no = 10;
	pl[3]->label[2].to_list = pl[1];
	pl[3]->label[3].label_no = 2;
	pl[3]->label[3].to_list = pl[5];

	/* List z = 0 */

	pl[4]->label[0].label_no = 4;
	pl[4]->label[0].to_list = pl[0];
	pl[4]->label[1].label_no = 0;
	pl[4]->label[1].to_list = pl[2];
	pl[4]->label[2].label_no = 1;
	pl[4]->label[2].to_list = pl[3];
	pl[4]->label[3].label_no = 7;
	pl[4]->label[3].to_list = pl[1];

	/* List z = 1 */

	pl[5]->label[0].label_no = 5;
	pl[5]->label[0].to_list = pl[0];
	pl[5]->label[1].label_no = 3;
	pl[5]->label[1].to_list = pl[2];
	pl[5]->label[2].label_no = 2;
	pl[5]->label[2].to_list = pl[3];
	pl[5]->label[3].label_no = 6;
	pl[5]->label[3].to_list = pl[1];


}

/*********************************************************
** Read n pictures  -- not called but here in case you need it
**********************************************************/
read_npict(picture)
float picture[JP][JP][JP];
{
	int i,j,k;
	int size = JP*JP*JP;            /* picture size */
	char indat[JP][JP][JP];         /* input data */
	for (i=0;i<10;i++) {
		fread(indat, size, 1, tp_02);
		pict_r++;
	}
}

/*********************************************************
** Read input data
**********************************************************/
read_data(picture)
float picture[JP][JP][JP];
{
	int i,j,k;
	int size = JP*JP*JP;            /* picture size */
	char indat[JP][JP][JP];         /* input data */
	printf("Reading in a frame. . .\r\n");
	fread(indat, size, 1, tp_02);
	if (feof(tp_02)) return;
	for (k = 0; k < JP; k++) {
		for (j = 0; j < JP; j++) {
			for (i = 0; i < JP; i++) {
				picture[i][j][k] = ABS((float) indat[i][j][k]);
			}
		}
	}
	printf("\t\t\t Done\n");
}

/*********************************************************
** find interpolation 
**********************************************************/
find_interpol(m)
int m;
{
	int i,j,k;
	float delta;
	delta = (float) m / (float) N_INTER;
	printf("Finding interpolation %d . . .\n",m);
	for (k = 0; k < JP; k++) {
		for (j = 0; j < JP; j++) {
			for (i = 0; i < JP; i++) {
				pict_int[i][j][k] = pict_old[i][j][k] * (1.0-delta)
				    + pict_new[i][j][k] * delta;
			}
		}
	}
	printf("Done\n");
}
/*********************************************************
** copy new picture into old picture 
**********************************************************/
cp_new_to_old(new,old)
float new[JP][JP][JP], old[JP][JP][JP];
{
	int i,j,k;
	for (k = 0; k < JP; k++) {
		for (j = 0; j < JP; j++) {
			for (i = 0; i < JP; i++) {
				old[i][j][k] = new[i][j][k];
			}
		}
	}
}

/***********************************************************
** Reads info to restart program
************************************************************/
read_to_restart()
{
	tp_01 = fopen("quepasa","r");
	fscanf(tp_01,"File with pictures = %s \n", in_file);
	fscanf(tp_01,"Contours (%d,%d,%d) \n",&c_lbl[0],&c_lbl[1],&c_lbl[2]);
	fscanf(tp_01,"Pictures read = %d \n",&pict_r);
	fscanf(tp_01,"Pictures done = %d \n",&pict_w);
	fscanf(tp_01,"Is this the first set of pictures ? %d",&pict_1);
	fclose(tp_01);
	printf("I will look in file %s\n",in_file);
	printf("I've already read %d pictures and done %d pictures\n",pict_r,pict_w);
	if(pict_1)printf("This is the first set of pictures\n");
	else printf("This is not the first set of pictures\n");

}

/***********************************************************
** Prints info to restart program
************************************************************/
save_to_restart()
{
	tp_01 = fopen("quepasa","w");
	fprintf(tp_01,"File with pictures = %s \n", in_file);
	fprintf(tp_01,"Contours (%d,%d,%d) \n",c_lbl[0],c_lbl[1],c_lbl[2]);
	fprintf(tp_01,"Pictures read = %d \n",pict_r);
	fprintf(tp_01,"Pictures done = %d \n",pict_w);
	fprintf(tp_01,"Is this the first set of pictures ? %d",pict_1);
	fclose(tp_01);
}

/**********************************************************
** Set graphics environment. Use mmode() to set up 
** projection and viewing matrices for lighting 
***********************************************************/
graph_environment()
{
	prefposition(-130,780,-50,600);  /* set window size */
	foreground();                    /* set foreground */
	winopen("PABLO's PLOT");         /* open window */
	RGBmode();                       /* set RGB mode on */
	gconfig();                       /* reconfigure */
	lsetdepth(0, 0x7FFFFF);          /* set 3-D viewport */
	zbuffer(TRUE);                   /* start z-buffer mode */
	mmode(MVIEWING);                 /* set current matrix mode */
	perspective(500,1.4,1.0,15.0);   /* perspective porj. trans. */
	loadmatrix(idmat);               /* load identity matrix */
	blendfunction(BF_SA, BF_MSA);    /* blend contour labels */
	blanktime(0);                    /* keep screen from blanking during record */
}

/******************************************************************
** def_light_calc()
** Tell the Graphics Library to DEFINE a simple lighting calculation
** that accounts for diffuse and ambient reflection. This simple
** lighting calculation happens to use the default lighting
** parameters in the Graphis Library.
********************************************************************/
def_light_calc()
{
	lmdef(DEFMATERIAL, 1, 21, material_1);
	lmdef(DEFMATERIAL, 3, 21, material_2);
	lmdef(DEFMATERIAL, 2, 21, material_3);
	lmdef(DEFMATERIAL, 4, 21, glass_material);
	lmdef(DEFLIGHT, 1, 0, NULL);
	lmdef(DEFLIGHT, 2, 14, light_1);
	lmdef(DEFLMODEL, 1, 0, NULL);
	lmdef(DEFLMODEL, 2, 10, ambient_1);
}

/*************************************************
** use_light_calc() 
** Tell the Graphics LIbrary to USE the simple 
** lighting calculation that we defined earlier.
***************************************************/
use_light_calc()
{
	lmbind(LIGHT1, 2);     /* use light source 2 */
	lmbind(LMODEL, 1);     /* use lighting model 1 */
}

/********************************************************
** Glass box 
*********************************************************/
glass_box()
{
	lmbind(MATERIAL, 4);
     
	n3f(&caja->donde[0]);
	bgnpolygon();
	v3f(&caja->esquina[0]);
	v3f(&caja->esquina[1]);
	v3f(&caja->esquina[3]);
	v3f(&caja->esquina[2]);
	endpolygon();

	n3f(&caja->donde[1]);
	bgnpolygon();
	v3f(&caja->esquina[4]);
	v3f(&caja->esquina[5]);
	v3f(&caja->esquina[7]);
	v3f(&caja->esquina[6]);
	endpolygon();

	n3f(&caja->donde[2]);
	bgnpolygon();
	v3f(&caja->esquina[0]);
	v3f(&caja->esquina[4]);
	v3f(&caja->esquina[5]);
	v3f(&caja->esquina[1]);
	endpolygon();

	n3f(&caja->donde[3]);
	bgnpolygon();
	v3f(&caja->esquina[1]);
	v3f(&caja->esquina[5]);
	v3f(&caja->esquina[7]);
	v3f(&caja->esquina[3]);
	endpolygon();

	n3f(&caja->donde[4]);
	bgnpolygon();
	v3f(&caja->esquina[3]);
	v3f(&caja->esquina[7]);
	v3f(&caja->esquina[6]);
	v3f(&caja->esquina[2]);
	endpolygon();

	n3f(&caja->donde[5]);
	bgnpolygon();
	v3f(&caja->esquina[2]);
	v3f(&caja->esquina[6]);
	v3f(&caja->esquina[4]);
	v3f(&caja->esquina[0]);
	endpolygon();

	linewidth(0);

}

/****************************************************
** Make box
*****************************************************/
make_box()
{
	int i;
	float scale = 3.0/(JP-1);
	float off_set = (JP-1)/2;
	caja = (BOX *) malloc(sizeof(BOX));

	caja->esquina[0].x = 0;
	caja->esquina[0].y = 0;
	caja->esquina[0].z = 0;

	caja->esquina[1].x = JP;
	caja->esquina[1].y = 0;
	caja->esquina[1].z = 0;

	caja->esquina[2].x = 0;
	caja->esquina[2].y = JP;
	caja->esquina[2].z = 0;

	caja->esquina[3].x = JP;
	caja->esquina[3].y = JP;
	caja->esquina[3].z = 0;

	caja->esquina[4].x = 0;
	caja->esquina[4].y = 0;
	caja->esquina[4].z = JP;

	caja->esquina[5].x = JP;
	caja->esquina[5].y = 0;
	caja->esquina[5].z = JP;

	caja->esquina[6].x = 0;
	caja->esquina[6].y = JP;
	caja->esquina[6].z = JP;

	caja->esquina[7].x = JP;
	caja->esquina[7].y = JP;
	caja->esquina[7].z = JP;

	caja->donde[0].x =  0.0;
	caja->donde[0].y =  0.0;
	caja->donde[0].z = -1.0;

	caja->donde[1].x =  0.0;
	caja->donde[1].y =  0.0;
	caja->donde[1].z =  1.0;

	caja->donde[2].x =  0.0;
	caja->donde[2].y = -1.0;
	caja->donde[2].z =  0.0;

	caja->donde[3].x =  1.0;
	caja->donde[3].y =  0.0;
	caja->donde[3].z =  0.0;

	caja->donde[4].x =  0.0;
	caja->donde[4].y =  1.0;
	caja->donde[4].z =  0.0;

	caja->donde[5].x = -1.0;
	caja->donde[5].y =  0.0;
	caja->donde[5].z =  0.0;

	for (i = 0; i < 8; i++) {
		caja->esquina[i].x =  scale * (caja->esquina[i].x - off_set);
		caja->esquina[i].y =  scale * (caja->esquina[i].y - off_set);
		caja->esquina[i].z =  scale * (caja->esquina[i].z - off_set);
	}
}
